package tapl5

import scala.io._

class TextFile(filename: String) extends Iterable[String] {

  def elements = new Iterator[String] {
    val in = Source.fromFile(filename)
    var line: Option[String] = nextLine

    private def nextLine: Option[String] = {
      val sb = new StringBuffer
      var ch = ' '
      while (in.hasNext && ch != '\n') {
        ch = in.next
        if (ch != '\n')
          sb.append(ch)
      }
      if (ch == '\n')
        Some(sb.toString)
      else
        None
    }

    def hasNext: Boolean = line match {
      case Some(_) => true
      case None    => false
    }

    def next = line match {
      case Some(s) => line = nextLine; s
      case None    => throw new NoSuchElementException()
    }
  }
}


trait TestSuite {
  type Output
  def parse(in: String): Option[Output]

  def showType(S: String): Unit;

  def checkString(line: String) = {
    try {
      Console.println("term: " + line)
      showType(line)
    } catch {
      case e: Throwable =>
        e.printStackTrace
    }
    Console.println("-------------------------------\n")
  }

  def check(filename: String) = try {
    val tf = new TextFile(filename)
    for (val line <- tf.elements; line.length != 0) {
      checkString(line)
    }
  } catch {
    case e: java.io.FileNotFoundException =>
      Console.println(e.getMessage)
      exit(1)
  }

}
